From 7c9ebfba6088a4c8c5294b35a7bc46a29919c65c Mon Sep 17 00:00:00 2001
From: webb <w@spiderden.org>
Date: Mon, 11 Sep 2023 09:41:38 -0400
Subject: [PATCH] Add client limits to response bodies and time.

---
 mastodon/apps.go     |  5 ++++-
 mastodon/mastodon.go | 30 +++++++++++++++++++++++++++++-
 2 files changed, 33 insertions(+), 2 deletions(-)

diff --git a/mastodon/apps.go b/mastodon/apps.go
index 5d925c3..ccab9bc 100644
--- a/mastodon/apps.go
+++ b/mastodon/apps.go
@@ -11,7 +11,7 @@ import (
 
 // AppConfig is a setting for registering applications.
 type AppConfig struct {
-	http.Client
+	*http.Client
 	Server     string
 	ClientName string
 
@@ -40,6 +40,9 @@ type Application struct {
 
 // RegisterApp returns the mastodon application.
 func RegisterApp(ctx context.Context, appConfig *AppConfig) (*Application, error) {
+	if appConfig.Client == nil {
+		appConfig.Client = client
+	}
 	params := url.Values{}
 	params.Set("client_name", appConfig.ClientName)
 	if appConfig.RedirectURIs == "" {
diff --git a/mastodon/mastodon.go b/mastodon/mastodon.go
index 8678314..5176f92 100644
--- a/mastodon/mastodon.go
+++ b/mastodon/mastodon.go
@@ -15,6 +15,7 @@ import (
 	"path"
 	"path/filepath"
 	"strings"
+	"time"
 
 	"github.com/tomnomnom/linkheader"
 )
@@ -170,10 +171,37 @@ func (c *Client) doAPI(ctx context.Context, method string, uri string, params in
 	return json.NewDecoder(resp.Body).Decode(&res)
 }
 
+type tripper struct {
+	underlying http.RoundTripper
+}
+
+func (t *tripper) RoundTrip(r *http.Request) (*http.Response, error) {
+	response, err := t.underlying.RoundTrip(r)
+	if response != nil && response.Body != nil {
+		response.Body = http.MaxBytesReader(nil, response.Body, 8000000)
+	}
+
+	return response, err
+}
+
+var client = &http.Client{
+	Transport: &tripper{
+		underlying: &http.Transport{
+			Proxy:                 http.ProxyFromEnvironment,
+			ForceAttemptHTTP2:     true,
+			MaxIdleConns:          100,
+			IdleConnTimeout:       90 * time.Second,
+			TLSHandshakeTimeout:   10 * time.Second,
+			ExpectContinueTimeout: 1 * time.Second,
+		},
+	},
+	Timeout: 8 * time.Second,
+}
+
 // NewClient return new mastodon API client.
 func NewClient(config *Config) *Client {
 	return &Client{
-		Client: http.DefaultClient,
+		Client: client,
 		config: config,
 	}
 }
-- 
2.42.0

